//
//  ViewController.swift
//  FrequencyCounter
//
//  Created by Phidgets on 2018-01-29.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {
    
    let ch = FrequencyCounter()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    @IBOutlet var countLabel: UILabel!
    @IBOutlet var timeLabel: UILabel!
    @IBOutlet var frequencyLabel: UILabel!
    @IBOutlet var totalCountLabel: UILabel!
    @IBOutlet var totalTimeLabel: UILabel!
    @IBOutlet var enabledStack: UIStackView!
    @IBOutlet var enabledSwitch: UISwitch!
    @IBOutlet var frequencyCutoffTextfield: UITextField!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var powerSupplySegmentedControl: UISegmentedControl!
    @IBOutlet var inputModeSegmentedControl: UISegmentedControl!
    @IBOutlet var daq1400Stack: UIStackView!
    @IBOutlet var stackView: UIStackView!
    
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.frequencyChange.addHandler(frequencychange_handler)
            let _ = ch.countChange.addHandler(countchange_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }

    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! FrequencyCounter
            
            let deviceID = try attachedDevice.getDeviceID()
            let minDataInterval = try Float(attachedDevice.getMinDataInterval())
            let maxDataInterval = try Float(attachedDevice.getMaxDataInterval())
            let dataInterval = try Float(attachedDevice.getDataInterval())
            let isEnabled = try attachedDevice.getEnabled()
            let frequencyCutoff = try attachedDevice.getFrequencyCutoff()
            
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = String(format: "%d ms", UInt32(dataInterval))
                self.frequencyCutoffTextfield.text = "\(frequencyCutoff)"
                self.enabledSwitch.isEnabled = isEnabled
            }
            
            if(deviceID == DeviceID.PN_1054){
                DispatchQueue.main.async {
                    self.enabledStack.isHidden = false
                    self.daq1400Stack.isHidden = true
                }
            }
            else if (deviceID == DeviceID.PN_DAQ1400){
                let powerSupply = try attachedDevice.getPowerSupply()
                let inputMode = try attachedDevice.getInputMode()
                DispatchQueue.main.async {
                    self.enabledStack.isHidden = true
                    self.daq1400Stack.isHidden = false
                    self.powerSupplySegmentedControl.selectedSegmentIndex = (powerSupply == PowerSupply.off) ? 0 : (powerSupply == PowerSupply.volts_12) ? 1 : 2
                    self.inputModeSegmentedControl.selectedSegmentIndex = (inputMode == InputMode.NPN) ? 0 : 1
                }
            }
            
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func frequencychange_handler(sender: FrequencyCounter, frequency: Double){
        DispatchQueue.main.async {
            self.frequencyLabel.text = String(format:"%f Hz", frequency)
        }
    }
    
    func countchange_handler(sender: FrequencyCounter, data: (counts: UInt64, timeChange: Double)){
        do{
            let totalCount = try ch.getCount()
            let totalTime = try ch.getTimeElapsed()
            DispatchQueue.main.async {
                self.countLabel.text = "\(data.counts)"
                self.timeLabel.text = "\(data.timeChange) ms"
                self.totalTimeLabel.text = "\(totalTime)"
                self.totalCountLabel.text = "\(totalCount)"
            }
        }catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }

}

